// Copyright 1998 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "OIAnimationInstance.h"

#import <Foundation/Foundation.h>
#import <OmniBase/OmniBase.h>
#import <OmniFoundation/OmniFoundation.h>

#import "OIAnimation.h"
#import "OIAnimationFrame.h"

RCS_ID("$Header: /Network/Developer/Source/CVS/OmniGroup/OIF/OIAnimationInstance.m,v 1.13 1998/12/09 22:25:32 kc Exp $")

@interface OIAnimationInstance (Private)
- (void)_displayNextFrame;
@end

@implementation OIAnimationInstance

static OFScheduler *animationScheduler;

+ (void)initialize;
{
    static BOOL initialized = NO;

    [super initialize];
    if (initialized)
        return;
    initialized = YES;

    animationScheduler = [[OFScheduler mainScheduler] subScheduler];
    [animationScheduler retain];
}

- initWithAnimation:(OIAnimation *)anAnimation;
{
    if (![super init])
        return nil;

    animation = [anAnimation retain];
    remainingLoops = [animation loopCount];

    return self;
}

- (void)dealloc;
{
    [frame removeObserver:self];
    [animation release];
    [super dealloc];
}

// Called by the animation, could possibly be in another thread.

- (void)animationEnded;
{
    if (remainingLoops == 0 || nextFrame <= 1)
        return;

    if (--remainingLoops > 0 && nextFrame != NSNotFound) {
        nextFrame = 0;
        [self _displayNextFrame];
    }
}

- (void)animationReceivedFrame:(OIAnimationFrame *)aFrame;
{
    if (nextFrame == NSNotFound)
        return;

    nextFrame++;
    if (frame != aFrame) {
        [frame removeObserver:self];
        frame = aFrame;
        if (!haveSize && [frame hasSize])
            [self setSize:[frame size]];
        [self updateImage:[frame image]];
        [frame addObserver:self];
    }

    if ([observerValues count])
        [animationScheduler scheduleSelector:@selector(_displayNextFrame) onObject:self withObject:nil afterTime:[frame delayInterval]];
}

// OIImage subclass

- (void)stopAnimation;
{
    nextFrame = NSNotFound;
}


- (void)addObserver:(id <OIImageObserver>)anObserver;
{
    BOOL jumpstartAnimation;

    jumpstartAnimation = [observerValues count] == 0;
    [super addObserver:anObserver];
    if (jumpstartAnimation) {
        nextFrame = 0;
        [self _displayNextFrame];
    }
}

// OIImageObserver protocol

- (void)imageDidSize:(OIImage *)anOmniImage;
{
    if (!haveSize)
        [self setSize:[anOmniImage size]];
}

- (void)imageDidUpdate:(OIImage *)anOmniImage;
{
    [self updateImage:[anOmniImage image]];
}

- (void)imageDidAbort:(OIImage *)anOmniImage;
{
}

// OWOptionalContent stuff

- (BOOL)shareable
{
    return NO;
}

@end

@implementation OIAnimationInstance (Private)

- (void)_displayNextFrame;
{
    if (nextFrame == NSNotFound)
        return;

    [animation animationInstance:self wantsFrame:nextFrame];
}

// Debugging

- (NSMutableDictionary *)debugDictionary;
{
    NSMutableDictionary *debugDictionary;

    debugDictionary = [super debugDictionary];
    [debugDictionary setObject:[NSNumber numberWithInt:remainingLoops] forKey:@"remainingLoops"];
    [debugDictionary setObject:[NSNumber numberWithInt:nextFrame] forKey:@"nextFrame"];

    return debugDictionary;
}

@end
